#include "area.h"

#include "options.h"

#include <QPainter>
#include <QPaintEvent>

#include <cstdlib>
#include <ctime>

Area::Area( int a, int b, int c, int d, QWidget *parent )
		:QWidget(parent), w(a), h(b), field_size(c), accuracy(d)
{
	createTab();
	
	srand( time(0) );
}

void Area::paintEvent( QPaintEvent * )
{
	QPainter painter( this );

	for( int i = 0 ; i<w ; i++ )
		for( int j = 0 ; j<h ; j++ )
		{
			QColor tmp(255, 255-(Tab[i][j]*255)/accuracy, 255-(Tab[i][j]*255)/accuracy );
			painter.setBrush( QBrush( tmp ) );
			painter.setPen( tmp );
			painter.drawRect(i*field_size, j*field_size, field_size, field_size );	
			//painter.fillRect(i*field_size, j*field_size, field_size, field_size, QBrush( tmp ) );	
		}
	
	painter.end();
}

void Area::placeOne()
{
	Tab[w/2][h/2] = accuracy;
}

void Area::calc()
{
	//srand( time(0) );
	
	float lx = options().lx;
	float rx = options().rx;
	float ty = options().ty;
	float dy = options().dy;
	int fade = options().fade;
	int burn = options().burn;

	for( int i = 1 ; i<w-1 ; i++ )
		for( int j = 1 ; j<h-1 ; j++ )
		{
			 if( Tab[i][j]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[i-1][j]*lx+Tab[i+1][j]*rx
					  +Tab[i][j-1]*dy+Tab[i][j+1]*ty)/4)
				 TmpTab[i][j] = accuracy;
			 else
				 TmpTab[i][j] = qMax( 0, Tab[i][j]-fade );
		}
	
	if( options().wrap == Options::none )
	{
		for( int j = 0 ; j<h ; j++ )
		{
			TmpTab[0][j] = options().none_value;
			TmpTab[w-1][j] = options().none_value;
		}

		for( int i = 1 ; i<w ; i++ )
		{
			TmpTab[i][0] = options().none_value;
			TmpTab[i][h-1] = options().none_value;
		}
	}
	else if( options().wrap == Options::torus )
	{
		for( int j = 0 ; j<h ; j++ )
		{
			 if( Tab[0][j]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[w-1][j]*lx+Tab[1%w][j]*rx
					  +Tab[0][mod(j-1, h)]*dy+Tab[0][(j+1)%h]*ty)/4)
				 TmpTab[0][j] = accuracy;
			 else
				 TmpTab[0][j] = qMax( 0, Tab[0][j]-fade );

			 if( Tab[w-1][j]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[qMax(w-2, 0)][j]*lx+Tab[0][j]*rx
					  +Tab[w-1][mod(j-1, h)]*dy+Tab[w-1][(j+1)%h]*ty)/4)
				 TmpTab[w-1][j] = accuracy;
			 else
				 TmpTab[w-1][j] = qMax( 0, Tab[w-1][j]-fade );
		}

		for( int i = 1 ; i<w-1 ; i++ )
		{
			 if( Tab[i][0]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[i-1][0]*lx+Tab[i+1][0]*rx
					  +Tab[i][h-1]*dy+Tab[i][1%h]*ty)/4)
				 TmpTab[i][0] = accuracy;
			 else
				 TmpTab[i][0] = qMax( 0, Tab[i][0]-fade );

			 if( Tab[i][h-1]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[i-1][h-1]*lx+Tab[i+1][h-1]*rx
					  +Tab[i][qMax(h-2, 0)]*dy+Tab[i][0]*ty)/4)
				 TmpTab[i][h-1] = accuracy;
			 else
				 TmpTab[i][h-1] = qMax( 0, Tab[i][h-1]-fade );
		}
	}
	else if( options().wrap == Options::strange_a )
	{
		for( int j = 1 ; j<h-1 ; j++ )
		{
			 if( Tab[0][j]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[j][0]*lx+Tab[1][j]*rx
					  +Tab[0][j-1]*dy+Tab[0][j+1]*ty)/4)
				 TmpTab[0][j] = accuracy;
			 else
				 TmpTab[0][j] = qMax( 0, Tab[0][j]-fade );

			 if( Tab[w-1][j]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[w-2][j]*lx+Tab[j][h-1]*rx
					  +Tab[w-1][j-1]*dy+Tab[w-1][j+1]*ty)/4)
				 TmpTab[w-1][j] = accuracy;
			 else
				 TmpTab[w-1][j] = qMax( 0, Tab[w-1][j]-fade );
		}

		for( int i = 1 ; i<w-1 ; i++ )
		{
			 if( Tab[i][0]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[i-1][0]*lx+Tab[i+1][0]*rx
					  +Tab[0][i]*dy+Tab[i][1]*ty)/4)
				 TmpTab[i][0] = accuracy;
			 else
				 TmpTab[i][0] = qMax( 0, Tab[i][0]-fade );

			 if( Tab[i][h-1]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[i-1][h-1]*lx+Tab[i+1][h-1]*rx
					  +Tab[i][h-2]*dy+Tab[w-1][i]*ty)/4)
				 TmpTab[i][h-1] = accuracy;
			 else
				 TmpTab[i][h-1] = qMax( 0, Tab[i][h-1]-fade );
		}
		
		//edges
		if( Tab[0][0]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
				(Tab[0][0]*lx+Tab[1][0]*rx
				+Tab[0][0]*dy+Tab[0][1]*ty)/4)
			TmpTab[0][0] = accuracy;
		else
			TmpTab[0][0] = qMax( 0, Tab[0][0]-fade );
		
		if( Tab[0][h-1]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
				(Tab[0][h-1]*lx+Tab[1][h-1]*rx
				+Tab[0][h-2]*dy+Tab[0][h-1]*ty)/4)
			TmpTab[0][h-1] = accuracy;
		else
			TmpTab[0][h-1] = qMax( 0, Tab[0][h-1]-fade );

		if( Tab[w-1][0]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
				(Tab[w-2][0]*lx+Tab[w-1][0]*rx
				+Tab[w-1][0]*dy+Tab[w-1][1]*ty)/4)
			TmpTab[w-1][0] = accuracy;
		else
			TmpTab[w-1][0] = qMax( 0, Tab[w-1][0]-fade );

		if( Tab[w-1][h-1]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
				(Tab[w-2][h-1]*lx+Tab[w-1][h-1]*rx
				+Tab[w-1][h-2]*dy+Tab[w-1][h-1]*ty)/4)
			TmpTab[w-1][h-1] = accuracy;
		else
			TmpTab[w-1][h-1] = qMax( 0, Tab[w-1][h-1]-fade );
	}
	else if( options().wrap == Options::strange_b )
	{
		for( int j = 1 ; j<h-1 ; j++ )
		{
			 if( Tab[0][j]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[j][h-1]*lx+Tab[1][j]*rx
					  +Tab[0][j-1]*dy+Tab[0][j+1]*ty)/4)
				 TmpTab[0][j] = accuracy;
			 else
				 TmpTab[0][j] = qMax( 0, Tab[0][j]-fade );

			 if( Tab[w-1][j]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[w-2][j]*lx+Tab[j][0]*rx
					  +Tab[w-1][j-1]*dy+Tab[w-1][j+1]*ty)/4)
				 TmpTab[w-1][j] = accuracy;
			 else
				 TmpTab[w-1][j] = qMax( 0, Tab[w-1][j]-fade );
		}

		for( int i = 1 ; i<w-1 ; i++ )
		{
			 if( Tab[i][0]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[i-1][0]*lx+Tab[i+1][0]*rx
					  +Tab[w-1][i]*dy+Tab[i][1]*ty)/4)
				 TmpTab[i][0] = accuracy;
			 else
				 TmpTab[i][0] = qMax( 0, Tab[i][0]-fade );

			 if( Tab[i][h-1]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
					 (Tab[i-1][h-1]*lx+Tab[i+1][h-1]*rx
					  +Tab[i][h-2]*dy+Tab[0][i]*ty)/4)
				 TmpTab[i][h-1] = accuracy;
			 else
				 TmpTab[i][h-1] = qMax( 0, Tab[i][h-1]-fade );
		}
		
		//edges
		if( Tab[0][0]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
				(Tab[0][0]*lx+Tab[1][0]*rx
				+Tab[0][0]*dy+Tab[0][1]*ty)/4)
			TmpTab[0][0] = accuracy;
		else
			TmpTab[0][0] = qMax( 0, Tab[0][0]-fade );
		
		if( Tab[0][h-1]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
				(Tab[0][h-1]*lx+Tab[1][h-1]*rx
				+Tab[0][h-2]*dy+Tab[0][h-1]*ty)/4)
			TmpTab[0][h-1] = accuracy;
		else
			TmpTab[0][h-1] = qMax( 0, Tab[0][h-1]-fade );

		if( Tab[w-1][0]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
				(Tab[w-2][0]*lx+Tab[w-1][0]*rx
				+Tab[w-1][0]*dy+Tab[w-1][1]*ty)/4)
			TmpTab[w-1][0] = accuracy;
		else
			TmpTab[w-1][0] = qMax( 0, Tab[w-1][0]-fade );

		if( Tab[w-1][h-1]<=burn && accuracy*(rand()/(RAND_MAX+1.0))<
				(Tab[w-2][h-1]*lx+Tab[w-1][h-1]*rx
				+Tab[w-1][h-2]*dy+Tab[w-1][h-1]*ty)/4)
			TmpTab[w-1][h-1] = accuracy;
		else
			TmpTab[w-1][h-1] = qMax( 0, Tab[w-1][h-1]-fade );
	}

	qSwap( Tab, TmpTab );
}

void Area::nextFrame()
{
	calc();
	update();
}

void Area::createTab()
{
	Tab = new f_type*[w];
	TmpTab = new f_type*[w];
	for( int i = 0 ; i<w ; i++ )
	{
		Tab[i] = new f_type[h];
		TmpTab[i] = new f_type[h];
		for( int j = 0 ; j<h ; j++ )
			Tab[i][j] = 0;
	}
}

void Area::destroyTab()
{
	for( int i = 0 ; i<w ; i++ )
	{
		delete[] Tab[i];
		delete[] TmpTab[i];
	}
	delete[] Tab;
	delete[] TmpTab;
}

void Area::clear()
{
	for( int i = 0 ; i<w ; i++ )
		for( int j = 0 ; j<h ; j++ )
			Tab[i][j] = 0;
}

void Area::changeSize( int a, int b, int c ,int d)
{
	destroyTab();	
	
	w = a;
	h = b;
	field_size = c;
	accuracy = d;
	
	createTab();
	
	adjustSize();
	update();
}
	
inline int Area::mod( int a, int b )
{
	if( a>=0 )
		return a%b;
	else
		return a+b;
}

int Area::getw()
{
	return w;
}

int Area::geth()
{
	return h;
}

int Area::getfield_size()
{
	return field_size;
}

int Area::getaccuracy()
{
	return accuracy;
}

QSize Area::minimumSizeHint() const
{
	return QSize( w*field_size, h*field_size);
}

QSize Area::sizeHint() const
{
	return QSize( w*field_size, h*field_size);
}

QSize Area::maximumSizeHint() const
{
	return QSize( w*field_size, h*field_size);
}

Area::~Area()
{
	destroyTab();
}
