#include "mainwindow.h"

#include "area.h"
#include "options.h"
#include "optionswindow.h"

#include <QtGui>

MainWindow::MainWindow()
{
	area = new Area( 100, 100, 4, 100, this );

	setCentralWidget(area);
	area->placeOne();
	area->update();
	
	nextFrameAct = new QAction(tr("Next frame"), this);
	nextFrameAct->setShortcut( Qt::Key_N );
	connect( nextFrameAct, SIGNAL( triggered() ), area, SLOT( nextFrame() ) );
	animateAct =  new QAction(tr("Animate"), this);
	animateAct->setShortcut( Qt::Key_A );
	animateAct->setCheckable( true );
	connect( animateAct, SIGNAL( changed() ), this, SLOT( animation() ) );
	
	QMenu *frames = menuBar()->addMenu(tr("&Frames"));
	frames->addAction(nextFrameAct);
	frames->addAction(animateAct);
	
	optionsAct = menuBar()->addAction(tr("&Options"));
	connect( optionsAct, SIGNAL( triggered() ), this, SLOT( optionsDialog() ) );

	setoneAct = menuBar()->addAction(tr("Clear"));
	connect( setoneAct, SIGNAL( triggered() ), area, SLOT( clear() ) );
	connect( setoneAct, SIGNAL( triggered() ), area, SLOT( placeOne() ) );
	connect( setoneAct, SIGNAL( triggered() ), area, SLOT( update() ) );
	
	QMenu *about = menuBar()->addMenu(tr("About"));
	aboutQtAct = about->addAction(tr("Qt library"), qApp, SLOT( aboutQt() ) );
	aboutAppAct = about->addAction(tr("This program") , this, SLOT( about() ) );
	//menuBar()->addAction(tr("Quit"), qApp, SLOT( quit() ) );


	setFixedSize( 400, 400+menuBar()->height() );
	setWindowTitle(tr("Fire simulation"));

	timer = new QTimer(this);
	timer->setInterval(options().internval);
	timer->setSingleShot( false );
	connect( timer, SIGNAL( timeout() ), area, SLOT( nextFrame() ) );
}

void MainWindow::optionsDialog()
{
	OptionsWindow w( area );
	if( w.exec() == QDialog::Accepted )
	{
		if( w.w->text().toInt() != area->getw() || 
				w.h->text().toInt() != area->geth() || 
				w.field_size->text().toInt() != area->getfield_size() || 
				w.accuracy->text().toInt() != area->getaccuracy() )
		{
			if( w.w->text().toInt()<3 || w.h->text().toInt()<3 )
			{
				QMessageBox::information(this, 
						tr("Width and height too small"),
						tr("<p>Widht and height must be at least three.</p>") );
			}
			area->changeSize( qMax(w.w->text().toInt(), 3), qMax(w.h->text().toInt(), 3),
				w.field_size->text().toInt(), w.accuracy->text().toInt() );
			
			setFixedSize( area->width(),area->height()+menuBar()->height() );
			area->placeOne();
			area->update();
		}

		timer->setInterval(options().internval);
	}
	
}

void MainWindow::animation()
{
	if( animateAct->isChecked() )
		timer->start(options().internval);
	else
		timer->stop();
}

void MainWindow::about()
{
	QMessageBox::about(this, tr("About fire simulation"),
			trUtf8( "<p>The <b>fire simulation</b> is a simply "
			    "example of a cellular automaton.</p>"
			    "<ul>"
			    "<li>Author:  Jacek Migdał (jacek@migdal.pl</li>"
			    "<li>License: GPL</li>"
			    "<li>Sources: http://jacek.migdal.pl/Fire_simulation/ </li>"
			    "<li>Release date: 10 May 2006</li>"
			    "</ul>"
			));
}
