#include <QPushButton>
#include <QRadioButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QGroupBox>
#include <QButtonGroup>
#include <QValidator>
#include <QLabel>
#include <QLineEdit>
#include <QMessageBox>

#include "area.h"
#include "options.h"

#include "optionswindow.h"

OptionsWindow::OptionsWindow( Area *a )
{
	QGroupBox *canvas = new QGroupBox(tr("Canvas:"));	
	QGroupBox *animate = new QGroupBox(tr("Animation options:"));	
	QGroupBox *algo = new QGroupBox(tr("Algorithm settings:"));	

	createCanvasSettings( canvas, a );
	createAnimationSettings( animate );
	createAlgorithmSettings( algo );
	
	QPushButton *ok = new QPushButton(tr("Save"));
	connect( ok, SIGNAL( clicked() ), this, SLOT( accept() ) );
	QPushButton *cancel = new QPushButton(tr("Cancel"));
	connect( cancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	
	QHBoxLayout *buttons = new QHBoxLayout;
	buttons->addWidget( ok );
	buttons->addWidget( cancel );
	
	QVBoxLayout *all = new QVBoxLayout;
	all->addWidget( canvas );
	all->addWidget( animate );
	all->addWidget( algo );
	all->addLayout( buttons );

	setLayout( all );

	setWindowTitle(tr("Config dialog"));
	connect( this, SIGNAL( accepted() ), this, SLOT( saveOptions() ) );
}

void OptionsWindow::createCanvasSettings( QWidget *box, Area *a )
{
	QVBoxLayout *label = new QVBoxLayout;	
	QVBoxLayout *lineedit = new QVBoxLayout;	

	QLabel *l_w = new QLabel(tr("Width:"));
	QLabel *l_h = new QLabel(tr("Height:"));
	QLabel *l_field_size = new QLabel(tr("Field size:"));
	QLabel *l_accuracy = new QLabel(tr("Max fire:"));
	
	label->addWidget( l_w );
	label->addWidget( l_h );
	label->addWidget( l_field_size );
	label->addWidget( l_accuracy );

	w = new QLineEdit( QString::number( a->getw() ) );
	h = new QLineEdit( QString::number( a->geth() ) );
	field_size = new QLineEdit( QString::number( a->getfield_size() ) );
	accuracy = new QLineEdit( QString::number( a->getaccuracy() ) );

	QIntValidator *val_int_a = new QIntValidator( 1, 10000, this );
	QIntValidator *val_int_b = new QIntValidator( 1, 255, this );
	w->setValidator( val_int_a );
	h->setValidator( val_int_a );
	field_size->setValidator( val_int_a );
	accuracy->setValidator( val_int_b );

	lineedit->addWidget( w );
	lineedit->addWidget( h );
	lineedit->addWidget( field_size );
	lineedit->addWidget( accuracy );
	
	QHBoxLayout *label_lineedit = new QHBoxLayout;
	label_lineedit->addLayout( label );
	label_lineedit->addLayout( lineedit );

	box->setLayout( label_lineedit );
}

void OptionsWindow::createAnimationSettings( QWidget *box )
{
	QHBoxLayout *main = new QHBoxLayout;

	QLabel *l_internval = new QLabel(tr("Frame delay in animation (in ms):"));	
	internval = new QLineEdit( QString::number( options().internval ) );
	QIntValidator *v_int = new QIntValidator( this );
	internval->setValidator( v_int );
	
	main->addWidget( l_internval );
	main->addWidget( internval );

	box->setLayout( main );
}

void OptionsWindow::createAlgorithmSettings( QWidget *box )
{
	QVBoxLayout *label = new QVBoxLayout;	
	QVBoxLayout *lineedit = new QVBoxLayout;	
	
	QLabel *l_lx = new QLabel(tr("Right spread ratio:"));
	QLabel *l_rx = new QLabel(tr("Left spread ratio:"));
	QLabel *l_ty = new QLabel(tr("Top spread ratio:"));
	QLabel *l_dy = new QLabel(tr("Bottom spread ratio:"));
	QLabel *l_fade = new QLabel(tr("Burn out ratio:"));
	QLabel *l_burn = new QLabel(tr("Level needed to catch fire:"));

	label->addWidget( l_rx );
	label->addWidget( l_lx );
	label->addWidget( l_ty );
	label->addWidget( l_dy );
	label->addWidget( l_fade );
	label->addWidget( l_burn );
	
	rx = new QLineEdit( QString::number( options().rx ) );
	lx = new QLineEdit( QString::number( options().lx ) );
	ty = new QLineEdit( QString::number( options().ty ) );
	dy = new QLineEdit( QString::number( options().dy ) );
	fade = new QLineEdit( QString::number( options().fade ) );
	burn = new QLineEdit( QString::number( options().burn ) );

	QDoubleValidator *v_double = new QDoubleValidator( this );
	QIntValidator *v_int = new QIntValidator( this );

	rx->setValidator( v_double );
	lx->setValidator( v_double );
	ty->setValidator( v_double );
	dy->setValidator( v_double );

	fade->setValidator( v_int );
	burn->setValidator( v_int );

	lineedit->addWidget( rx );
	lineedit->addWidget( lx );
	lineedit->addWidget( ty );
	lineedit->addWidget( dy );
	lineedit->addWidget( fade );
	lineedit->addWidget( burn );

	QHBoxLayout *label_lineedit = new QHBoxLayout;
	label_lineedit->addLayout( label );
	label_lineedit->addLayout( lineedit );

	QButtonGroup *group = new QButtonGroup;
	none = new QRadioButton(tr("No wrap"));
	torus = new QRadioButton(tr("Torus wrap - top-bottom and right-left"));
	strange_a = new QRadioButton(tr("Strange wrap A - top-left and bottom-right"));
	strange_b = new QRadioButton(tr("Strange wrap B - top-right and bottom-left"));
	
	group->addButton( none );
	group->addButton( torus );
	group->addButton( strange_a );
	group->addButton( strange_b );

	QLabel *l_none_value = new QLabel(tr("Constant value: "));
	none_value = new QLineEdit( QString::number( options().none_value ) );
	none_value->setValidator( v_int );
	QHBoxLayout *box_none_value = new QHBoxLayout;
	box_none_value->addWidget( l_none_value );
	box_none_value->addWidget( none_value );
	
	switch( options().wrap )
	{
		case Options::none:
		none->toggle();
		break;

		case Options::torus:
		torus->toggle();
		none_value->setDisabled( true );
		break;

		case Options::strange_a:
		strange_a->toggle();
		none_value->setDisabled( true );
		break;

		case Options::strange_b:
		strange_b->toggle();
		none_value->setDisabled( true );
		break;
	}

	connect( group, SIGNAL( buttonClicked(QAbstractButton *) ),
		 this, SLOT( checkConstant(QAbstractButton *) ) );

	QVBoxLayout *last = new QVBoxLayout;
	last->addLayout( label_lineedit );
	last->addWidget( none );
	last->addLayout( box_none_value );
	last->addWidget( torus );
	last->addWidget( strange_a );
	last->addWidget( strange_b );
	
	box->setLayout( last );
}

void OptionsWindow::checkConstant( QAbstractButton *button )
{
	if( button == none )
		none_value->setDisabled( false );
	else
		none_value->setDisabled( true );
}

void OptionsWindow::saveOptions()
{
	options().rx = rx->text().toDouble();
	options().lx = lx->text().toDouble();
	options().ty = ty->text().toDouble();
	options().dy = dy->text().toDouble();

	options().internval = internval->text().toInt();
	
	options().fade = fade->text().toInt();
	options().burn = burn->text().toInt();

	if( none->isChecked() )
	{
		options().wrap = Options::none;
		options().none_value = none_value->text().toInt();
	}
	else if ( torus->isChecked() )
		options().wrap = Options::torus;
	else
	{
		if( h->text().toInt() != w->text().toInt() )
		{
			QMessageBox::information(this, 
					tr("Width is not equal height"),
					tr("<p>For strange wraps it is necessery to set " \
					   "the same width and height.<p>" \
					   "<p>Torus wrap is set.</p> ") );
			options().wrap = Options::torus;
		}
		else if( strange_a->isChecked() )
			options().wrap = Options::strange_a;
		else if( strange_b->isChecked() )
			options().wrap = Options::strange_b;
	}
}
